@@ -5,7 +5,7 @@ using namespace PSQN;
 
 context("testing intrapolate class") {
   test_that("intrapolate gives the correct result with a 2nd order poly") {
-    constexpr double const f0 = 0,
+    constexpr double f0 = 0,
                            d0 = -1,
                            x0 = 0,
                             f = 3.75,
@@ -18,7 +18,7 @@ context("testing intrapolate class") {
   }
 
   test_that("intrapolate gives the correct result with a 3rd order poly") {
-    constexpr double const f0 = 0,
+    constexpr double f0 = 0,
                            d0 = -1,
                            x0 = 0,
                          fold = 5.3125,
